#!/usr/bin/perl -w 
#
# Author: Christian Monstein
# Created: 2014-05-16
# Updated: 2014-06-13, 2017-01-10, 2017-04-06, 2017-07-02, 2018-12-10, 2021-05-14
# Compresses FIT-file to reduce newtork load
# Copies FIT-files to the FTP-server at FHNW in Switzerland
# Move FIT files into a local backup device (local, USB, external,...)
# This script should be execute every 15 minutes by e.g. ssfree.exe or crontab at minutes 01, 16, 31 and 46
# Delay time might be increased for slow network connections (1...20)

use strict;
use warnings;
use File::Path;
use File::Copy;
use Net::FTP;
use IO::Compress::Gzip qw(gzip $GzipError);

my $delay = 2; # Delay time to allow FTP-upload, before renaming it


my @tnow  = localtime(time());
my $today = sprintf('%04i%02i%02i', $tnow[5]+1900, $tnow[4]+1, $tnow[3]);

my $SourcePath = 'c:/CALLISTO-01/FITfiles'; # where the FIT-files are stored from CALLISTO
my $DestinationPath = 'c:/CALLISTO-01/FITbackup'; # where the zipped FIT-files are saved
  
opendir(ID, $SourcePath) || die("Couldn't find directory"); 
my @listing = readdir(ID);
closedir(ID);

foreach my $listing(sort @listing) 
{
     my $filename = $listing;
     if ($filename =~ /^XTOWN_(\d\d\d\d)(\d\d)(\d\d)_.*\.fit?$/)
     {
	my ($year, $mon, $day) = ($1, $2, $3);
	if ( ! -d "$DestinationPath/$year/$mon/$day") 
	{
	   mkpath("$DestinationPath/$year/$mon/$day");
	}
	my $old = "$SourcePath/$listing";
	gzip $old => $old.'.gz' or die "gzip failed: $GzipError\n"; # Perform GZIP
	my $zip = $old.'.gz';
	my $ftp = Net::FTP->new("147.86.8.73", Timeout => 500, Passive => 1, Debug   => 0);
	$ftp->login("username","password") or die 'Cannot login ', $ftp->message;	
	$ftp->cwd('/') or die 'Cannot change folder ', $ftp->message;	
	$ftp->binary();
	my $tmp = $filename . ".gz.tmp"; # upload as .tmp to avoid blocking at server site
	$ftp->put($zip,$tmp) or die $ftp->message;
	sleep($delay);
	$ftp->rename($tmp,$filename.'.gz') or die $ftp->message; # once uploaded, rename to original filename
	$ftp->quit;
	#print "\nSource: ",$old," -> FTP-server copied.  ";
	my $new = "$DestinationPath/$year/$mon/$day";
	move($zip ,$new); # backup on a local device
	unlink $SourcePath.'/'.$filename; # delete original FIT-file after backup
     }
}
#print "\n\nFTP finished...\n";
sleep(1);








